@@ -126,6 +126,10 @@ static int icmp6_open_router_common(const struct icmp6_filter *filter,
 				ifr.ifr_name, strlen(ifr.ifr_name) + 1);
 	}
 
+	if (r < 0)
+		goto fail;
+
+	r = setsockopt(s, SOL_SOCKET, SO_TIMESTAMP, &yes, sizeof(yes));
 	if (r < 0)
 		goto fail;
 
@@ -191,7 +195,8 @@ static int icmp6_send_router_solicitation(int s, const uint8_t mac[static 6])
 	return 0;
 }
 
-static int icmp6_receive(int s, void *buf, size_t buf_len, struct in6_addr *src)
+static int icmp6_receive(int s, void *buf, size_t buf_len, struct in6_addr *src,
+				uint64_t *out_timestamp)
 {
 	char c_msg_buf[CMSG_SPACE(sizeof(int))];
 	struct iovec iov = {
@@ -210,6 +215,7 @@ static int icmp6_receive(int s, void *buf, size_t buf_len, struct in6_addr *src)
 	};
 	struct cmsghdr *cmsg;
 	ssize_t l;
+	uint64_t timestamp = 0;
 
 	l = recvmsg(s, &msg, MSG_DONTWAIT);
 	if (l < 0)
@@ -230,10 +236,16 @@ static int icmp6_receive(int s, void *buf, size_t buf_len, struct in6_addr *src)
 
 			if (hops != 255)
 				return -EMULTIHOP;
+		} else if (cmsg->cmsg_level == SOL_SOCKET &&
+				cmsg->cmsg_type == SCM_TIMESTAMP) {
+			const struct timeval *tv = (void *) CMSG_DATA(cmsg);
+
+			timestamp = tv->tv_sec * L_USEC_PER_SEC + tv->tv_usec;
 		}
 	}
 
 	memcpy(src, &saddr.sin6_addr, sizeof(saddr.sin6_addr));
+	*out_timestamp = timestamp ?: l_time_now();
 	return 0;
 }
 
@@ -344,23 +356,25 @@ static void icmp6_client_setup_routes(struct l_icmp6_client *client)
 static int icmp6_client_handle_message(struct l_icmp6_client *client,
 						struct nd_router_advert *ra,
 						size_t len,
-						const struct in6_addr *src)
+						const struct in6_addr *src,
+						uint64_t timestamp)
 {
-	struct l_icmp6_router *r = _icmp6_router_parse(ra, len, src->s6_addr);
+	struct l_icmp6_router *r =
+		_icmp6_router_parse(ra, len, src->s6_addr, timestamp);
 
 	if (!r)
 		return -EBADMSG;
 
+	icmp6_client_event_notify(client,
+					L_ICMP6_CLIENT_EVENT_ROUTER_FOUND,
+					r);
+
+	/* DHCP6 client may have stopped us */
+	if (!client->io)
+		return -ECANCELED;
+
 	if (!client->ra) {
 		client->ra = r;
-		icmp6_client_event_notify(client,
-					L_ICMP6_CLIENT_EVENT_ROUTER_FOUND,
-					NULL);
-
-		/* DHCP6 client may have stopped us */
-		if (!client->ra)
-			return -ECANCELED;
-
 		icmp6_client_setup_routes(client);
 		return 0;
 	}
@@ -369,8 +383,7 @@ static int icmp6_client_handle_message(struct l_icmp6_client *client,
 	 * TODO: Figure out if the RA has updated info and update routes
 	 * accordingly.
 	 */
-	_icmp6_router_free(client->ra);
-	client->ra = r;
+	_icmp6_router_free(r);
 	return 0;
 }
 
@@ -382,6 +395,7 @@ static bool icmp6_client_read_handler(struct l_io *io, void *userdata)
 	ssize_t l;
 	struct in6_addr src;
 	int r;
+	uint64_t timestamp = 0;
 
 	/* Poke to see how many bytes we need to read / alloc */
 	l = recv(s, NULL, 0, MSG_PEEK|MSG_TRUNC);
@@ -392,7 +406,7 @@ static bool icmp6_client_read_handler(struct l_io *io, void *userdata)
 	}
 
 	ra = l_malloc(l);
-	if (icmp6_receive(s, ra, l, &src) < 0)
+	if (icmp6_receive(s, ra, l, &src, &timestamp) < 0)
 		goto done;
 
 	if ((size_t) l < sizeof(struct nd_router_advert)) {
@@ -400,7 +414,7 @@ static bool icmp6_client_read_handler(struct l_io *io, void *userdata)
 		goto done;
 	}
 
-	r = icmp6_client_handle_message(client, ra, l, &src);
+	r = icmp6_client_handle_message(client, ra, l, &src, timestamp);
 	if (r < 0)
 		goto done;
 
@@ -661,7 +675,8 @@ void _icmp6_router_free(struct l_icmp6_router *r)
 
 struct l_icmp6_router *_icmp6_router_parse(const struct nd_router_advert *ra,
 						size_t len,
-						const uint8_t src[static 16])
+						const uint8_t src[static 16],
+						uint64_t timestamp)
 {
 	struct l_icmp6_router *r;
 	const uint8_t *opts;
@@ -726,6 +741,7 @@ struct l_icmp6_router *_icmp6_router_parse(const struct nd_router_advert *ra,
 	if (r->pref == 0x2) /* If invalid, reset to medium */
 		r->pref = 0;
 
+	r->start_time = timestamp;
 	r->lifetime = L_BE16_TO_CPU(ra->nd_ra_router_lifetime);
 
 	opts = (uint8_t *) (ra + 1);
@@ -798,3 +814,20 @@ LIB_EXPORT bool l_icmp6_router_get_other(const struct l_icmp6_router *r)
 
 	return r->other;
 }
+
+LIB_EXPORT uint16_t l_icmp6_router_get_lifetime(const struct l_icmp6_router *r)
+{
+	if (unlikely(!r))
+		return false;
+
+	return r->lifetime;
+}
+
+/* Get the reception timestamp, i.e. when lifetime is counted from */
+LIB_EXPORT uint64_t l_icmp6_router_get_start_time(const struct l_icmp6_router *r)
+{
+	if (unlikely(!r))
+		return false;
+
+	return r->start_time;
+}
