@@ -55,6 +55,7 @@
 #include "src/nl80211util.h"
 
 #define DPP_FRAME_MAX_RETRIES 5
+#define DPP_FRAME_RETRY_TIMEOUT 1
 
 static uint32_t netdev_watch;
 static struct l_genl_family *nl80211;
@@ -138,12 +139,40 @@ struct dpp_sm {
 	uint32_t connect_scan_id;
 	uint64_t frame_cookie;
 	uint8_t frame_retry;
+	void *frame_pending;
+	size_t frame_size;
+	struct l_timeout *retry_timeout;
 
 	struct l_dbus_message *pending;
 
 	bool mcast_support : 1;
+	bool roc_started : 1;
 };
 
+static void *dpp_serialize_iovec(struct iovec *iov, size_t iov_len,
+				size_t *out_len)
+{
+	unsigned int i;
+	size_t size = 0;
+	uint8_t *ret;
+
+	for (i = 0; i < iov_len; i++)
+		size += iov[i].iov_len;
+
+	ret = l_malloc(size);
+	size = 0;
+
+	for (i = 0; i < iov_len; i++) {
+		memcpy(ret + size, iov[i].iov_base, iov[i].iov_len);
+		size += iov[i].iov_len;
+	}
+
+	if (out_len)
+		*out_len = size;
+
+	return ret;
+}
+
 static void dpp_free_auth_data(struct dpp_sm *dpp)
 {
 	if (dpp->own_proto_public) {
@@ -209,6 +238,16 @@ static void dpp_reset(struct dpp_sm *dpp)
 		dpp->peer_asn1 = NULL;
 	}
 
+	if (dpp->frame_pending) {
+		l_free(dpp->frame_pending);
+		dpp->frame_pending = NULL;
+	}
+
+	if (dpp->retry_timeout) {
+		l_timeout_remove(dpp->retry_timeout);
+		dpp->retry_timeout = NULL;
+	}
+
 	dpp->state = DPP_STATE_NOTHING;
 	dpp->new_freq = 0;
 	dpp->frame_retry = 0;
@@ -250,9 +289,10 @@ static void dpp_free(struct dpp_sm *dpp)
 static void dpp_send_frame_cb(struct l_genl_msg *msg, void *user_data)
 {
 	struct dpp_sm *dpp = user_data;
+	int err = l_genl_msg_get_error(msg);
 
-	if (l_genl_msg_get_error(msg) < 0) {
-		l_error("Error sending frame");
+	if (err < 0) {
+		l_error("Error sending frame (%d)", err);
 		return;
 	}
 
@@ -267,6 +307,19 @@ static void dpp_send_frame(struct dpp_sm *dpp,
 {
 	struct l_genl_msg *msg;
 
+	/*
+	 * A received frame could potentially come in after the ROC session has
+	 * ended. In this case the frame needs to be stored until ROC is started
+	 * and sent at that time. The offchannel_id is also checked since
+	 * this is not applicable when DPP is in a responder role waiting
+	 * on the currently connected channel i.e. offchannel is never used.
+	 */
+	if (!dpp->roc_started && dpp->offchannel_id) {
+		dpp->frame_pending = dpp_serialize_iovec(iov, iov_len,
+							&dpp->frame_size);
+		return;
+	}
+
 	msg = l_genl_msg_new_sized(NL80211_CMD_FRAME, 512);
 	l_genl_msg_append_attr(msg, NL80211_ATTR_WDEV, 8, &dpp->wdev_id);
 	l_genl_msg_append_attr(msg, NL80211_ATTR_WIPHY_FREQ, 4, &freq);
@@ -281,6 +334,20 @@ static void dpp_send_frame(struct dpp_sm *dpp,
 	}
 }
 
+static void dpp_frame_retry(struct dpp_sm *dpp)
+{
+	struct iovec iov;
+
+	iov.iov_base = dpp->frame_pending;
+	iov.iov_len = dpp->frame_size;
+
+	dpp_send_frame(dpp, &iov, 1, dpp->current_freq);
+
+	l_free(dpp->frame_pending);
+	dpp->frame_pending = NULL;
+}
+
+
 static size_t dpp_build_header(const uint8_t *src, const uint8_t *dest,
 				enum dpp_frame_type type,
 				uint8_t buf[static 32])
@@ -1294,7 +1361,7 @@ static void dpp_presence_announce(struct dpp_sm *dpp)
 	dpp_send_frame(dpp, iov, 2, dpp->current_freq);
 }
 
-static void dpp_send_authenticate_request(struct dpp_sm *dpp)
+static bool dpp_send_authenticate_request(struct dpp_sm *dpp)
 {
 	uint8_t hdr[32];
 	uint8_t attrs[256];
@@ -1305,6 +1372,12 @@ static void dpp_send_authenticate_request(struct dpp_sm *dpp)
 	struct station *station = station_find(netdev_get_ifindex(dpp->netdev));
 	struct scan_bss *bss = station_get_connected_bss(station);
 
+	/* Got disconnected by the time the peer was discovered */
+	if (!bss) {
+		dpp_reset(dpp);
+		return false;
+	}
+
 	l_ecc_point_get_data(dpp->own_proto_public, i_proto_key,
 				sizeof(i_proto_key));
 
@@ -1337,6 +1410,8 @@ static void dpp_send_authenticate_request(struct dpp_sm *dpp)
 	iov[1].iov_len = ptr - attrs;
 
 	dpp_send_frame(dpp, iov, 2, dpp->current_freq);
+
+	return true;
 }
 
 static void dpp_roc_started(void *user_data)
@@ -1352,6 +1427,20 @@ static void dpp_roc_started(void *user_data)
 	 *   the authenticate response now.
 	 */
 
+	dpp->roc_started = true;
+
+	/*
+	 * The retry timer indicates a frame was not acked in which case we
+	 * should not change any state or send any frames until that expires.
+	 */
+	if (dpp->retry_timeout)
+		return;
+
+	if (dpp->frame_pending) {
+		dpp_frame_retry(dpp);
+		return;
+	}
+
 	switch (dpp->state) {
 	case DPP_STATE_PRESENCE:
 		if (dpp->role == DPP_CAPABILITY_CONFIGURATOR)
@@ -1401,6 +1490,7 @@ static void dpp_presence_timeout(int error, void *user_data)
 	struct dpp_sm *dpp = user_data;
 
 	dpp->offchannel_id = 0;
+	dpp->roc_started = false;
 
 	/*
 	 * If cancelled this is likely due to netdev going down or from Stop().
@@ -1948,7 +2038,8 @@ static void dpp_handle_presence_announcement(struct dpp_sm *dpp,
 
 	dpp->state = DPP_STATE_AUTHENTICATING;
 
-	dpp_send_authenticate_request(dpp);
+	if (!dpp_send_authenticate_request(dpp))
+		return;
 
 	/*
 	 * Should we wait for an ACK then go offchannel?
@@ -2005,6 +2096,25 @@ static bool match_wdev(const void *a, const void *b)
 	return *wdev_id == dpp->wdev_id;
 }
 
+static void dpp_frame_timeout(struct l_timeout *timeout, void *user_data)
+{
+	struct dpp_sm *dpp = user_data;
+
+	l_timeout_remove(timeout);
+	dpp->retry_timeout = NULL;
+
+	/*
+	 * ROC has not yet started (in between an ROC timeout and starting a
+	 * new session), this will most likely result in the frame failing to
+	 * send. Just bail out now and the roc_started callback will take care
+	 * of sending this out.
+	 */
+	if (!dpp->roc_started)
+		return;
+
+	dpp_frame_retry(dpp);
+}
+
 static void dpp_mlme_notify(struct l_genl_msg *msg, void *user_data)
 {
 	struct dpp_sm *dpp;
@@ -2043,11 +2153,19 @@ static void dpp_mlme_notify(struct l_genl_msg *msg, void *user_data)
 		return;
 	}
 
-	l_debug("No ACK from peer, re-transmitting");
+	/* This should never happen */
+	if (L_WARN_ON(dpp->frame_pending))
+		return;
+
+	l_debug("No ACK from peer, re-transmitting in %us",
+			DPP_FRAME_RETRY_TIMEOUT);
 
 	dpp->frame_retry++;
 
-	dpp_send_frame(dpp, &iov, 1, dpp->current_freq);
+	dpp->frame_pending = l_memdup(iov.iov_base, iov.iov_len);
+	dpp->frame_size = iov.iov_len;
+	dpp->retry_timeout = l_timeout_create(DPP_FRAME_RETRY_TIMEOUT,
+						dpp_frame_timeout, dpp, NULL);
 }
 
 static void dpp_unicast_notify(struct l_genl_msg *msg, void *user_data)
@@ -2427,6 +2545,8 @@ static struct l_dbus_message *dpp_start_configurator_common(
 
 		if (!dpp->mcast_support)
 			dpp->state = DPP_STATE_AUTHENTICATING;
+
+		dpp->new_freq = bss->frequency;
 	} else
 		dpp->current_freq = bss->frequency;
 
