@@ -85,6 +85,7 @@ struct network {
 	bool is_hs20:1;
 	bool anqp_pending:1;	/* Set if there is a pending ANQP request */
 	bool owe_hidden_pending:1;
+	bool provisioning_hidden:1;
 	uint8_t transition_disable; /* Temporary cache until info is set */
 	bool have_transition_disable:1;
 	int rank;
@@ -188,6 +189,8 @@ void network_connected(struct network *network)
 				network_secret_check_cacheable, network);
 
 	l_queue_clear(network->blacklist, NULL);
+
+	network->provisioning_hidden = false;
 }
 
 void network_disconnected(struct network *network)
@@ -195,6 +198,9 @@ void network_disconnected(struct network *network)
 	network_settings_close(network);
 
 	l_queue_clear(network->blacklist, NULL);
+
+	if (network->provisioning_hidden)
+		station_hide_network(network->station, network);
 }
 
 /* First 64 entries calculated by 1 / pow(n, 0.3) for n >= 1 */
@@ -1243,6 +1249,9 @@ static void passphrase_callback(enum agent_result result,
 
 err:
 	network_settings_close(network);
+
+	if (network->provisioning_hidden)
+		station_hide_network(station, network);
 }
 
 static struct l_dbus_message *network_connect_psk(struct network *network,
@@ -1661,8 +1670,10 @@ struct l_dbus_message *network_connect_new_hidden_network(
 
 	switch (network_get_security(network)) {
 	case SECURITY_PSK:
+		network->provisioning_hidden = true;
 		return network_connect_psk(network, bss, message);
 	case SECURITY_NONE:
+		network->provisioning_hidden = true;
 		station_connect_network(station, network, bss, message);
 		return NULL;
 	default:
