@@ -32,6 +32,7 @@ ssize_t write_file(const void *buffer, size_t len, bool preserve_times,
 			const char *path_fmt, ...)
 	__attribute__((format(printf, 4, 5)));
 
+bool storage_is_file(const char *filename);
 bool storage_create_dirs(void);
 void storage_cleanup_dirs(void);
 char *storage_get_path(const char *format, ...);
@@ -49,3 +50,13 @@ int storage_network_remove(enum security type, const char *ssid);
 
 struct l_settings *storage_known_frequencies_load(void);
 void storage_known_frequencies_sync(struct l_settings *known_freqs);
+
+int __storage_decrypt(struct l_settings *settings, const char *ssid,
+				bool *changed);
+char *__storage_encrypt(const struct l_settings *settings, const char *ssid,
+				size_t *out_len);
+bool storage_decrypt(struct l_settings *settings, const char *path,
+			const char *name);
+
+bool storage_init(const uint8_t *key, size_t key_len);
+void storage_exit(void);
